@@ -2,9 +2,10 @@
 
 from dataclasses import dataclass
 from datetime import datetime
-from typing import List, Optional
+from typing import List, Optional, Union
 
 import apibara.application.indexer_service_pb2 as indexer_service_pb2
+from apibara.starknet import get_selector_from_name
 
 
 @dataclass
@@ -15,6 +16,11 @@ class Topic:
     def from_proto(p: indexer_service_pb2.Topic):
         return Topic([c.value for c in p.choices])
 
+    def to_proto(self) -> indexer_service_pb2.Topic:
+        return indexer_service_pb2.Topic(
+            choices=[indexer_service_pb2.TopicValue(value=c) for c in self.choices]
+        )
+
     def __str__(self) -> str:
         choices = [f"0x{c.hex()}" for c in self.choices]
         return f'Topic({"|".join(choices)})'
@@ -22,9 +28,26 @@ class Topic:
 
 @dataclass
 class EventFilter:
+    name: Optional[str]
     address: Optional[bytes]
     topics: List[Topic]
 
+    @classmethod
+    def from_event_name(
+        cls, name: str, address: Optional[Union[str, bytes]]
+    ) -> "EventFilter":
+        """Create an EventFilter from the event name."""
+        if isinstance(address, str):
+            address = bytes.fromhex(address.replace("0x", ""))
+        topic_hex = hex(get_selector_from_name(name))[2:]
+        # fromhex requires an even number of digits
+        if len(topic_hex) % 2 == 1:
+            topic_hex = "0" + topic_hex
+        topic_value = bytes.fromhex(topic_hex)
+        topics = [Topic([topic_value])]
+
+        return cls(name, address, topics)
+
     @staticmethod
     def from_proto(p: indexer_service_pb2.EventFilter):
         address = None
@@ -32,7 +55,12 @@ class EventFilter:
             address = p.address
 
         topics = [Topic.from_proto(t) for t in p.topics]
-        return EventFilter(address, topics)
+        return EventFilter(name=None, address=address, topics=topics)
+
+    def to_proto(self) -> indexer_service_pb2.EventFilter:
+        return indexer_service_pb2.EventFilter(
+            address=self.address, topics=[t.to_proto() for t in self.topics]
+        )
 
     def __str__(self) -> str:
         address = ""
@@ -87,6 +115,7 @@ class BlockHeader:
 
 @dataclass
 class Event:
+    name: Optional[str]
     address: bytes
     block_index: int
     topics: List[bytes]
@@ -96,10 +125,13 @@ class Event:
     def from_proto(p: indexer_service_pb2.Event):
         topics = [bytes(t.value) for t in p.topics]
         data = [bytes(d.value) for d in p.data]
-        return Event(bytes(p.address), p.block_index, topics, data)
+        return Event(None, bytes(p.address), p.block_index, topics, data)
 
     def __str__(self) -> str:
-        return f"Event(address=0x{self.address.hex()}, block_index={self.block_index}, ...{len(self.topics)} topics, ...{len(self.data)} data)"
+        name = ""
+        if self.name is not None:
+            name = f"name={self.name}, "
+        return f"Event({name}address=0x{self.address.hex()}, block_index={self.block_index}, ...{len(self.topics)} topics, ...{len(self.data)} data)"
 
 
 @dataclass
